/*
 * Copyright (c) 1999-2019 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_IMAGE_HPP
#define PW_CORE_IMAGE_HPP

#include <pw/core/globals.hpp>
#include <pw/core/size.hpp>

namespace pw {

class image {
public:

    image() = default;

    enum pixel_layout {
        RGB8,
        RGBA8,
        LUM
    };

    image(const sizei& s, pixel_layout t, void *ptr = nullptr);

    bool create(const sizei& s, pixel_layout t, void *ptr = nullptr);

    void destroy(bool release_memory = false);

	const uint8_t *data() const { return _data.data(); }
	uint8_t *data() { return _data.data(); }

    pixel_layout layout() const;
    void set_layout(const pixel_layout &layout);

    uint64_t change_count() const;
    void set_change_count(const uint64_t &change_count);

    static uint32_t bytes_per_pixel(pixel_layout t);

	typedef shared_ptr<image> ptr;

    sizei size() const;

protected:

    sizei _size;
    pixel_layout _layout;
    uint64_t _change_count;

    std::vector<uint8_t> _data;

};

}

#endif
