#include "pw/core/mesh.hpp"

namespace pw {

mesh::mesh()
{
}

void mesh::set_vertices(const mesh::vertex3array_t &v)
{
	// first set vertices
	_vertices = v;

	// update bounds
	compute_bounds();
}

void mesh::apply(const matrix4x4 &m)
{
	// apply transformation to all vertices
    for (auto &v : _vertices)
		v = vector4(m * v.homogenous()).project();

	// recompute bounds
	compute_bounds();
}

void mesh::compute_bounds()
{
	// only do if there are vertices
	if (_vertices.size()) {

		// reset
		_aabb.max = _aabb.min = _vertices.front();

		// update
		for (auto const & v : _vertices) {
			_aabb.max.x() = std::max(v.x(),_aabb.max.x());
			_aabb.max.y() = std::max(v.y(),_aabb.max.y());
			_aabb.max.z() = std::max(v.z(),_aabb.max.z());
			_aabb.min.x() = std::min(v.x(),_aabb.min.x());
			_aabb.min.y() = std::min(v.y(),_aabb.min.y());
			_aabb.min.z() = std::min(v.z(),_aabb.min.z());
		}
	}
}

}
